clc
clear
close all

global beta alpha gamma v_o m_tau n_tau tau


beta=0.9;
alpha=0.36;
gamma=0.5;

 
tau_m=1/alpha*gamma/(1+gamma)*(1-alpha*beta);

tau_o=fsolve(@(tau)(alpha*beta-alpha)/(1-(1-tau)*alpha*beta-alpha*tau)+gamma/tau-alpha*beta*(1+gamma)/(1-alpha*beta)/(1-tau),0.5);

q = @(s,tau) log(1-alpha*tau-s)+gamma*log(alpha*tau)+alpha*beta*(1+gamma)/(1-alpha*beta)*log(s);
  
v_o=q((1-tau_o)*alpha*beta,tau_o)/(1-beta);

m_tau=1/(1-beta)*(log(1-(1-tau_o)*alpha*beta-alpha*tau_o)+gamma*log(alpha*tau_o)+alpha*beta*(1+gamma)/(1-alpha*beta)*log((1-tau_o)*alpha*beta));

n_tau=alpha*(1+gamma)/(1-alpha*beta);
 
tau_initial=fsolve(@fun_tau_initial,0.5);

 
x0=[alpha*beta,gamma*alpha];

x1=fsolve(@fun_ss_r,x0);

tau_r=x1(2)/alpha;


n_t=100;

grid_t=linspace(0.9*tau_o, 1.02*tau_m,n_t)';


p=zeros(n_t,1);

for i=1:n_t
    
    tau=grid_t(i);
    
    x_initial=[1-(1-tau)*alpha*beta-alpha*tau,1-(1-tau)*alpha*beta-alpha*tau,tau]; 
    
    x=fsolve(@fun_tau,x_initial);
    
    p(i)=x(3);
     
end





figure1 = figure;

% Create axes
axes1 = axes('Parent',figure1);
% Uncomment the following line to preserve the X-limits of the axes


xlim(axes1,[grid_t(1) grid_t(n_t)]);
ylim(axes1,[0.3000    0.7000]);
box(axes1,'on');
hold(axes1,'all'); 

% Create multiple lines using matrix input to plot
plot1 = plot(grid_t,[p,grid_t],'Parent',axes1);
set(plot1(1),'LineWidth',3,'Color',[   0.8500    0.3250    0.0980]);
set(plot1(2),'LineWidth',1 ,'LineStyle','--','Color',[0    0.4470    0.74100],'DisplayName','45 line');

 


yL = get(gca,'YLim');
line([tau_m tau_m],yL,'LineStyle','--','Color','k');
line([tau_initial,tau_initial],yL,'LineStyle','--','Color','k');
line([tau_r tau_r],yL,'LineStyle','--','Color','k');
line([tau_o tau_o],yL,'LineStyle','--','Color','k');

set(gca,'TickLabelInterpreter', 'latex','FontSize',14);
xticks([tau_r tau_o tau_initial tau_m])
xticklabels({'$$\tau^R$$','$$\tau^*$$','$$\tau_0$$','$$\tau^M$$'})
yticks([] )

xlabel('$$\tau_t$$','interpreter','latex','FontSize',14);
ylabel('$\tau_{t+1}$','interpreter','latex','FontSize',14);


I = legend('$$q^*(\tau)$$','45 degree line');
set(I,'interpreter','latex'); %set Latex interpreter
set(I,'FontWeight','bold','FontSize',14,'Location','NorthWest');
set(I,'FontName','Times New Roman');
set(I,'box','Off');
 