clc
clear
close all

global alpha beta gamma_g gamma_c gamma_l mu_1 mu_2 delta Indicator tau_kf tau_nf tau_f r_bar Reduction

alpha=0.36;
beta=0.96;
gamma_g=0.08475;
gamma_c=0.3*(1-gamma_g);
gamma_l=1-gamma_c-gamma_g;
delta=0.08;
mu_1=1-delta;
mu_2=delta;

Reduction=1;     % Whether allow depreciation tax reduction
Indicator=3;     % What tax instrument is used
Table=zeros(4,7);



%**************************************************************************
%   Pareto
%**************************************************************************

s_po=mu_2*alpha*beta/(1-beta*mu_1);
temp=(1-mu_1*beta-mu_2*alpha*beta)/(1-beta*mu_1)/(1-alpha)*gamma_l/(gamma_c+gamma_g);
n_po=1/(1+temp);
k_po=(s_po/delta*n_po^(1-alpha))^(1/(1-alpha));
y_po=k_po^alpha*n_po^(1-alpha);
c_po=(y_po-s_po*y_po)*gamma_c/(gamma_c+gamma_g);
i_po=s_po*y_po;
g_po=y_po-i_po-c_po;
r_po=alpha*y_po/k_po;


Table(1,1)=y_po/y_po;
Table(1,2)=k_po/y_po;
Table(1,3)=c_po/y_po;
Table(1,4)=g_po/y_po;
Table(1,5)=c_po/g_po;
Table(1,6)=n_po;


%**************************************************************************
%   Organization
%**************************************************************************

r_bar=0.1;

for loop=1:30
    
    [tau_x fval] = fminbnd(@fun_reduction,0,1); 
    
    if (Indicator==1)
        
        tau_n = tau_x;
        tau_k = 0;
        tau   = 0;
        
    elseif (Indicator==2)
        
        tau_n = 0;
        tau_k = tau_x;
        tau   = 0;
        
    elseif (Indicator==3)
        
        tau_n = 0;
        tau_k = 0;
        tau   = tau_x;
        
    end
    
    
    if (Reduction==1)
        mu=(1-alpha)*(tau_n+tau)+alpha*(tau_k+tau)*(1-delta/r_bar);
        s_oe=mu_2*alpha*beta*(1-(tau_k+tau)*(1-delta/r_bar))/(1-beta*mu_1);
    
    elseif (Reduction==0)
        
        mu=(1-alpha)*(tau_n+tau)+alpha*(tau_k+tau);
        s_oe=mu_2*alpha*beta*(1-(tau_k+tau))/(1-beta*mu_1);
        
    end
         
    temp=(1-alpha)*(1-tau_n-tau)/(1-s_oe-mu)*gamma_c/gamma_l;
    n_oe=temp/(1+temp); 
    
    k_oe=(s_oe/delta*n_oe^(1-alpha))^(1/(1-alpha));
    y_oe=k_oe^alpha*n_oe^(1-alpha);
    c_oe=y_oe*(1-s_oe-mu);
    g_oe=y_oe*mu;
    r_oe=alpha*y_oe/k_oe;
    
    r_bar=r_oe;
    
end



Table(4,1)=y_oe/y_po;
Table(4,2)=k_oe/y_oe;
Table(4,3)=c_oe/y_oe;
Table(4,4)=g_oe/y_oe;
Table(4,5)=c_oe/g_oe;
Table(4,6)=n_oe;
Table(4,7)=tau_x;

%**************************************************************************
%   Markov
%**************************************************************************



tau_nf = tau_x;
tau_kf = tau_x;
tau_f  = tau_x;

r_bar=0.1;

for loop=1:150 
    
    for i=1:10
        
        [tau_x fval] = fminbnd(@fun_markov_reduction,0,2);
        
        
        if (Indicator==1)
            
            tau_nf = tau_x;
            tau_kf = 0;
            tau_f  = 0;
            
        elseif (Indicator==2)
            
            tau_nf = 0;
            tau_kf = tau_x;
            tau_f  = 0;
            
        elseif (Indicator==3)
            
            tau_nf = 0;
            tau_kf = 0;
            tau_f  = tau_x;
            
        end      
     
    end
    
    
    if (Indicator==1)
        
        tau_n = tau_x;
        tau_k = 0;
        tau   = 0;
        
    elseif (Indicator==2)
        
        tau_n = 0;
        tau_k = tau_x;
        tau   = 0;
        
    elseif (Indicator==3)
        
        tau_n = 0;
        tau_k = 0;
        tau   = tau_x;
        
    end
    
    
    if (Reduction==1)
        
        mu=(1-alpha)*(tau_n+tau)+alpha*(tau_k+tau)*(1-delta/r_bar);
        s_m=mu_2*alpha*beta*(1-(tau_k+tau)*(1-delta/r_bar))/(1-beta*mu_1);
    
    elseif (Reduction==0)
        
        mu=(1-alpha)*(tau_n+tau)+alpha*(tau_k+tau);
        s_m=mu_2*alpha*beta*(1-(tau_k+tau))/(1-beta*mu_1);
        
    end    
     
    temp=(1-alpha)*(1-tau_n-tau)/(1-s_m-mu)*gamma_c/gamma_l;
    n_m=temp/(1+temp); 
    
    
    k_m=(s_m/delta*n_m^(1-alpha))^(1/(1-alpha));
    y_m=k_m^alpha*n_m^(1-alpha);
    c_m=y_m*(1-s_m-mu);
    g_m=y_m*mu; 
    
    r_m=alpha*y_m/k_m;
    
    lambda=0.1;
    r_bar=r_m*lambda+r_bar*(1-lambda);
      
end


Table(3,1)=y_m/y_po;
Table(3,2)=k_m/y_m;
Table(3,3)=c_m/y_m;
Table(3,4)=g_m/y_m;
Table(3,5)=c_m/g_m;
Table(3,6)=n_m;
Table(3,7)=tau_x;



%**************************************************************************
%   Ramsey
%**************************************************************************


r_bar=0.1;

if (Reduction==0)
    
    n_loop=1;
    
elseif (Reduction==1)
    
    n_loop=20;
    
end


for loop=1:n_loop
    
    syms m s
    

    if (Indicator==1) 
        
        tau_n=m/(1-alpha);
        tau_k=0;
        tau  =0;
        
    elseif (Indicator==2) 
        
        tau_n=0;       
        tau_k=m/alpha/(1-delta/r_bar);
        tau  =0;
        
        if (Reduction==0)
            
            tau_k=m/alpha;
            
        end
        
    elseif (Indicator==3) 
        
        tau_n=0;       
        tau_k=0;
        tau  =m/(1-alpha*delta/r_bar);
        
     
        if (Reduction==0)
            
            tau=m;
            
        end
            
    end
     
        
    temp=gamma_c/gamma_l*(1-alpha)*(1-tau_n-tau)/(1-s-m);
    n=temp/(1+temp);
        
    R1=1/beta*s/(1-s-m);    
    R2=(delta*alpha*(1-(tau_k+tau)*(1-delta/r_bar))+(1-delta)*s)/(1-s-m);
    
    if (Reduction==0)
            
        R2=(delta*alpha*(1-(tau_k+tau))+(1-delta)*s)/(1-s-m);
            
    end        
        
    f=(gamma_c+gamma_g)/(1-(1-delta+alpha*delta)*beta)*(alpha*beta*delta*log(s)+(1-alpha)*(1-beta*(1-delta))*log(n))+...
            gamma_c*log(1-s-m)+gamma_l*log(1-n)+gamma_g*log(m);
        
        
    a1=simplify(diff(f,m)/diff(f,s)-(diff(R2,m)-beta*diff(R1,m))/(diff(R2,s)-beta*diff(R1,s)));
    a2=simplify(R1-R2);
        
    save a1 a1
    save a2 a2
        
    x_initial=[0.2,0.15];
    x=fsolve(@fun_ramsey,x_initial);
        
    m_r=x(1);
    s_r=x(2);
    


    if (Indicator==1) 
        
        tau_n=m_r/(1-alpha); 
        tau  =0;
        tau_k=0;
        tau_x=tau_n;
        
        
    elseif (Indicator==2) 
        
        tau_n=0; 
        tau_k=m_r/alpha/(1-delta/r_bar);
        tau  =0;
        
        if (Reduction==0)
            
            tau_k=m_r/alpha;
            
        end
        
        tau_x=tau_k;
        
    elseif (Indicator==3) 
        
        tau_n=0;    
        tau_k=0;
        tau  =m_r/(1-alpha*delta/r_bar); 
        
        
        if (Reduction==0)
            
            tau=m_r;
            
        end
        
        tau_x=tau;
        
    end    
    
    
    temp=gamma_c/gamma_l*(1-alpha)*(1-tau_n-tau)/(1-s_r-m_r);
    n_r=temp/(1+temp);
    
    k_r=(s_r/delta*n_r^(1-alpha))^(1/(1-alpha));
    y_r=k_r^alpha*n_r^(1-alpha);
    g_r=y_r*m_r;
    c_r=y_r*(1-s_r-m_r);
    
    r_r=alpha*y_r/k_r;
    
    r_bar=r_r;
    
end


Table(2,1)=y_r/y_po;
Table(2,2)=k_r/y_r;
Table(2,3)=c_r/y_r;
Table(2,4)=g_r/y_r;
Table(2,5)=c_r/g_r;
Table(2,6)=n_r;
Table(2,7)=tau_x;

Table=Table';